"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AnchorProgramSdk = void 0;
const transaction_1 = require("./transaction");
class AnchorProgramSdk {
    constructor(args) {
        this.program = args.program;
        this.provider = args.provider;
    }
    get programId() {
        return this.program.programId;
    }
    get rpc() {
        return this.program.rpc;
    }
    get instruction() {
        return this.program.instruction;
    }
    get wallet() {
        return this.provider.wallet;
    }
    get account() {
        return this.program.account;
    }
    get errors() {
        var _a;
        return (_a = this.program.idl.errors) === null || _a === void 0 ? void 0 : _a.reduce((acc, err) => {
            acc.set(err.code, `${err.name}: ${err.msg}`);
            return acc;
        }, new Map());
    }
    sendInstructions(instructions, signers, payer) {
        return (0, transaction_1.sendInstructions)(this.errors || new Map(), this.provider, instructions, signers, payer);
    }
    execute(command, payer = this.wallet.publicKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield command;
            if (instructions.length > 0) {
                yield this.sendInstructions(instructions, signers, payer);
            }
            return output;
        });
    }
    executeBig(command, payer = this.wallet.publicKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield command;
            if (instructions.length > 0) {
                yield (0, transaction_1.sendMultipleInstructions)(this.errors || new Map(), this.provider, instructions, signers, payer);
            }
            return output;
        });
    }
}
exports.AnchorProgramSdk = AnchorProgramSdk;
//# sourceMappingURL=anchorProgramSdk.js.map