"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SplTokenMetadata = exports.getImageFromMeta = void 0;
const common_1 = require("@project-serum/common");
const web3_js_1 = require("@solana/web3.js");
const _1 = require(".");
const arweave_1 = require("./arweave");
const metadata_1 = require("./metadata");
const USE_CDN = false; // copied from metaplex. Guess support isn't there yet?
const routeCDN = (uri) => {
    let result = uri;
    if (USE_CDN) {
        result = uri.replace("https://arweave.net/", "https://coldcdn.com/api/cdn/bronil/");
    }
    return result;
};
function getImageFromMeta(meta) {
    var _a, _b;
    if (meta === null || meta === void 0 ? void 0 : meta.image) {
        return meta === null || meta === void 0 ? void 0 : meta.image;
    }
    else {
        const found = (_b = (((_a = meta === null || meta === void 0 ? void 0 : meta.properties) === null || _a === void 0 ? void 0 : _a.files) || []).find((f) => typeof f !== "string" && f.type === metadata_1.MetadataCategory.Image)) === null || _b === void 0 ? void 0 : _b.uri;
        return found;
    }
}
exports.getImageFromMeta = getImageFromMeta;
const imageFromJson = (newUri, extended) => {
    const image = getImageFromMeta(extended);
    if (image) {
        const file = image.startsWith("http")
            ? extended.image
            : `${newUri}/${extended.image}`;
        return routeCDN(file);
    }
};
class SplTokenMetadata {
    constructor(opts) {
        this.provider = opts.provider;
    }
    static init(provider) {
        return __awaiter(this, void 0, void 0, function* () {
            return new this({
                provider,
            });
        });
    }
    static getArweaveMetadata(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (uri) {
                const newUri = routeCDN(uri);
                const cached = localStorage.getItem(newUri);
                if (cached) {
                    return JSON.parse(cached);
                }
                else {
                    try {
                        // TODO: BL handle concurrent calls to avoid double query
                        const result = yield fetch(newUri);
                        let data = yield result.json();
                        if (data.uri) {
                            data = Object.assign(Object.assign({}, data), (yield SplTokenMetadata.getArweaveMetadata(data.uri)));
                        }
                        try {
                            localStorage.setItem(newUri, JSON.stringify(data));
                        }
                        catch (e) {
                            // ignore
                        }
                        return data;
                    }
                    catch (e) {
                        console.log(`Could not fetch from ${uri}`, e);
                        return undefined;
                    }
                }
            }
        });
    }
    static getImage(uri) {
        return __awaiter(this, void 0, void 0, function* () {
            if (uri) {
                const newUri = routeCDN(uri);
                const metadata = yield SplTokenMetadata.getArweaveMetadata(uri);
                // @ts-ignore
                if (metadata === null || metadata === void 0 ? void 0 : metadata.uri) {
                    // @ts-ignore
                    return getImage(metadata === null || metadata === void 0 ? void 0 : metadata.uri);
                }
                return imageFromJson(newUri, metadata);
            }
        });
    }
    static getEdition(tokenMint) {
        return __awaiter(this, void 0, void 0, function* () {
            return (yield web3_js_1.PublicKey.findProgramAddress([
                Buffer.from(metadata_1.METADATA_PREFIX),
                new web3_js_1.PublicKey(_1.METADATA_PROGRAM_ID).toBuffer(),
                tokenMint.toBuffer(),
                Buffer.from(metadata_1.EDITION),
            ], new web3_js_1.PublicKey(_1.METADATA_PROGRAM_ID)))[0];
        });
    }
    getEditionInfo(metadata) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!metadata) {
                return {};
            }
            const editionKey = yield SplTokenMetadata.getEdition(new web3_js_1.PublicKey(metadata.mint));
            let edition;
            let masterEdition;
            const editionOrMasterEditionAcct = yield this.provider.connection.getAccountInfo(editionKey);
            const editionOrMasterEdition = editionOrMasterEditionAcct
                ? (editionOrMasterEditionAcct === null || editionOrMasterEditionAcct === void 0 ? void 0 : editionOrMasterEditionAcct.data[0]) == _1.MetadataKey.EditionV1
                    ? (0, metadata_1.decodeEdition)(editionOrMasterEditionAcct.data)
                    : (0, _1.decodeMasterEdition)(editionOrMasterEditionAcct.data)
                : null;
            if (editionOrMasterEdition instanceof metadata_1.Edition) {
                edition = editionOrMasterEdition;
                const masterEditionInfoAcct = yield this.provider.connection.getAccountInfo(new web3_js_1.PublicKey(editionOrMasterEdition.parent));
                masterEdition =
                    masterEditionInfoAcct &&
                        (0, _1.decodeMasterEdition)(masterEditionInfoAcct.data);
            }
            else {
                masterEdition = editionOrMasterEdition;
            }
            return {
                edition,
                masterEdition: masterEdition || undefined,
            };
        });
    }
    getMetadata(metadataKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataAcc = yield this.provider.connection.getAccountInfo(metadataKey);
            return metadataAcc && (0, metadata_1.decodeMetadata)(metadataAcc.data);
        });
    }
    getTokenMetadata(metadataKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadataAcc = yield this.provider.connection.getAccountInfo(metadataKey);
            const metadata = metadataAcc && (0, metadata_1.decodeMetadata)(metadataAcc.data);
            const data = yield SplTokenMetadata.getArweaveMetadata(metadata === null || metadata === void 0 ? void 0 : metadata.data.uri);
            const image = yield SplTokenMetadata.getImage(metadata === null || metadata === void 0 ? void 0 : metadata.data.uri);
            const description = data === null || data === void 0 ? void 0 : data.description;
            const mint = metadata &&
                (yield (0, common_1.getMintInfo)(this.provider, new web3_js_1.PublicKey(metadata.mint)));
            return Object.assign({ metadata: metadata || undefined, metadataKey,
                image, mint: mint || undefined, data,
                description }, (metadata ? yield this.getEditionInfo(metadata) : {}));
        });
    }
    sendInstructions(instructions, signers, payer) {
        return (0, _1.sendInstructions)(new Map(), this.provider, instructions, signers, payer);
    }
    presignCreateArweaveUrlInstructions({ name, symbol, description = "", image, creators, files = [], payer = this.provider.wallet.publicKey, env = "mainnet-beta", uploadUrl = arweave_1.ARWEAVE_UPLOAD_URL, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const metadata = {
                name,
                symbol,
                description,
                image,
                externalUrl: "",
                animationUrl: undefined,
                properties: {
                    category: metadata_1.MetadataCategory.Image,
                    files,
                },
                creators: creators ? creators : null,
                sellerFeeBasisPoints: 0,
            };
            const realFiles = yield (0, arweave_1.getFilesWithMetadata)(files, metadata);
            const prepayTxnInstructions = yield (0, arweave_1.prePayForFilesInstructions)(payer, realFiles, uploadUrl, env);
            return {
                instructions: prepayTxnInstructions,
                signers: [],
                output: {
                    files: realFiles,
                },
            };
        });
    }
    presignCreateArweaveUrl(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { output: { files }, instructions, signers, } = yield this.presignCreateArweaveUrlInstructions(args);
            const txid = yield this.sendInstructions(instructions, signers);
            return {
                files,
                txid,
            };
        });
    }
    getArweaveUrl({ txid, mint, files = [], uploadUrl = arweave_1.ARWEAVE_UPLOAD_URL, env = "mainnet-beta", }) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, arweave_1.uploadToArweave)(txid, mint, files, uploadUrl, env);
            const metadataFile = (_a = result.messages) === null || _a === void 0 ? void 0 : _a.find((m) => m.filename === "manifest.json");
            if (!metadataFile) {
                throw new Error("Metadata file not found");
            }
            // Use the uploaded arweave files in token metadata
            return `https://arweave.net/${metadataFile.transactionId}`;
        });
    }
    createMetadataInstructions({ data, authority = this.provider.wallet.publicKey, mint, payer = this.provider.wallet.publicKey, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const instructions = [];
            const metadata = yield (0, metadata_1.createMetadata)(data, authority === null || authority === void 0 ? void 0 : authority.toBase58(), mint.toBase58(), payer.toBase58(), instructions, payer.toBase58());
            return {
                instructions,
                signers: [],
                output: {
                    metadata: new web3_js_1.PublicKey(metadata),
                },
            };
        });
    }
    createMetadata(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.createMetadataInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
    updateMetadataInstructions({ data, authority, metadata, }) {
        return __awaiter(this, void 0, void 0, function* () {
            const instructions = [];
            const metadataAcct = yield this.getMetadata(metadata);
            yield (0, _1.updateMetadata)(data == null
                ? undefined
                : typeof data === "undefined"
                    ? metadataAcct === null || metadataAcct === void 0 ? void 0 : metadataAcct.data
                    : data, authority == null
                ? undefined
                : typeof authority === "undefined"
                    ? metadataAcct === null || metadataAcct === void 0 ? void 0 : metadataAcct.updateAuthority
                    : authority.toBase58(), undefined, metadataAcct.mint, metadataAcct.updateAuthority, instructions, metadata.toBase58());
            return {
                instructions,
                signers: [],
                output: {
                    metadata,
                },
            };
        });
    }
    updateMetadata(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { instructions, signers, output } = yield this.updateMetadataInstructions(args);
            yield this.sendInstructions(instructions, signers, args.payer);
            return output;
        });
    }
}
exports.SplTokenMetadata = SplTokenMetadata;
//# sourceMappingURL=splTokenMetadata.js.map